# -*- coding: utf-8 -*-
##------------------------------------------
##   My Library CSV file treatment  Ver 0.08
##
##               2022.06.29 Masahiro Izutsu
##------------------------------------------
## my_csv.py        (mylib_csv.py  Ver 0.04)
##      Ver 0.05    2022.06.29      ログ出力変更
##      Ver 0.06    2022.09.14      機能拡張
##      Ver 0.07    2022.09.18      ログ関連変更
##      Ver 0.08    2022.10.14      list3_csv追加

import my_logging
import csv

# CSV data
#           col_0   col_1   ... col_n
#   row_0    d00     d01         d0n
#   row_1    d10     d11         d1n
#     :
#   row_n    dn0     dn1         dnn

class CSVtreatment:
    logger = None
    encoding = ''
    path = ''
    
    # 初期化
    #   logsel: ログ出力選択
    def __init__(self, path, encoding = 'utf_8_sig', logsel = 4):
        self.logger = my_logging.get_module_logger_sel(__name__, logsel)
        self.path = path 
        self.encoding = encoding

    def _get_logger_object(self):
        return self.logger

    # CSV ファイル書き込み  in:[] 文字列リスト
    def write_csv(self, row_data):
        try:
            with open(self.path, 'w', encoding = self.encoding) as f:
                writer = csv.writer(f, lineterminator='\n')
                writer.writerow(row_data)
        except FileNotFoundError as e:
            self.logger.error(e)
        except csv.Error as e:
            self.logger.debug(e)

    # CSV ファイル追加書き込み  in:[] 文字列リスト
    def append_csv(self, row_data):
        try:
            with open(self.path, 'a', encoding = self.encoding) as f:
                writer = csv.writer(f, lineterminator='\n')
                writer.writerow(row_data)
        except FileNotFoundError as e:
            self.logger.error(e)
        except csv.Error as e:
            self.logger.debug(e)

    # CSV ファイル読み出し  out: 文字列
    def read_csv(self):
        try:
            with open(self.path, 'r', encoding = self.encoding) as f:
                string = f.read()
        except FileNotFoundError as e:
            string = ''
            self.logger.debug(e)
        except csv.Error as e:
            string = ''
            self.logger.debug(e)
            
        return string

    # CSV ファイルチェック key(col_0) = value(d0X) があるかを調べる
    #   out: True(あり)/False(なし)
    #   ※ 存在しない key を引数にしてはいけない !!
    def check_csv(self, key, value):
        s = self.read_csv_row(key, value)
        return s != None

    # CSV ファイル key(col_0) = value(d0X) 列の column値の読み出し
    #   out: value列のcolumn文字列  (None = 存在しない)
    #   ※ 存在しない key を引数にしてはいけない !!
    def read_csv_row_value(self, key, value, column):
        s = self.read_csv_row(key, value)
        s0 = s[column] if s is not None else None
        return s0

    # CSV ファイル key(col_0) = value(d0X) 列の読み出し
    #   out: 列の文字列  (None = 存在しない)
    #   ※ 存在しない key を引数にしてはいけない !!
    def read_csv_row(self, key, value):
        try:
            s = None
            with open(self.path, 'r', encoding = self.encoding) as f:
                reader = csv.DictReader(f)
                for row in reader:              # 一致する最後の行
                    if row[key] == value:
                        s = row
        except FileNotFoundError as e:
            s = None
            self.logger.error(e)
        except csv.Error as e:
            s = None
            self.logger.exception(e)
        return s

    # CSV ファイル 文字列リスト2読み出し
    #   out: 文字列リスト1, 文字列リスト2   []= ファイルがない
    #
    def read_list2_csv(self):
        befor = []
        after = []
        try:
            with open(self.path, 'r', encoding = self.encoding) as f:
                reader = csv.reader(f)
                for row in reader:
                    befor.append(str(row[0]))
                    after.append(str(row[1]))
        except FileNotFoundError as e:
            pass

        except csv.Error as e:
            self.logger.exception(e)
            
        return befor, after

    # CSV ファイル 文字列リスト3読み出し
    #   out: 文字列リスト1, 文字列リスト2, 文字列リスト3   []= ファイルがない
    #
    def read_list3_csv(self):
        list1 = []
        list2 = []
        list3 = []
        try:
            with open(self.path, 'r', encoding = self.encoding) as f:
                reader = csv.reader(f)
                for row in reader:
                    list1.append(str(row[0]))
                    list2.append(str(row[1]))
                    list3.append(str(row[2]))
        except FileNotFoundError as e:
            pass

        except IndexError as e:
            pass

        except csv.Error as e:
            self.logger.exception(e)
            
        return list1, list2, list3

    # CSV ファイル 文字列リスト2書き込み
    #   in: 文字列リスト1, 文字列リスト2
    #
    def write_list2_csv(self, befor, after):
        sf = True
        for b, a in zip(befor, after):
            row_data = [b, a]                   # 登録データ
            if sf:                              #  初回
                self.write_csv(row_data)
                sf = False
            else:                               #  2回目以降
                self.append_csv(row_data)

    # CSV ファイル 文字列リスト3書き込み
    #   in: 文字列リスト1, 文字列リスト2, 文字列リスト3
    #
    def write_list3_csv(self, list1, list2, list3):
        sf = True
        for ls1, ls2, ls3 in zip(list1, list2, list3):
            row_data = [ls1, ls2, ls3]          # 登録データ
            if sf:                              #  初回
                self.write_csv(row_data)
                sf = False
            else:                               #  2回目以降
                self.append_csv(row_data)


#-----Test routine-----
# $ python3 my_csv.py
#
if __name__ == "__main__":
    import my_print
    
    logsel = 1
    logselm = 1
    loggerm = my_logging.get_module_logger_sel('mycsv', logselm)

    path = 'mylib_test/sample_csv_utf8sig.csv'
    encoding = 'utf_8_sig'
    my_csv_treatment = CSVtreatment(path, encoding, logsel)

    ## 1
    my_print.printY('\nTest 1: write_csv()')
    row_data = ['INDEX', '姓', '名']
    my_csv_treatment.write_csv(row_data)
    loggerm.info('write_csv')

    ## 2
    my_print.printY('\nTest 2: append_csv()')
    row_data = [1, '井筒', '政弘']
    my_csv_treatment.append_csv(row_data)
    loggerm.info('append_csv')

    ## 3
    my_print.printY('\nTest 3: append_csv()')
    row_data = [2, '新田', '孝']
    my_csv_treatment.append_csv(row_data)
    loggerm.info('append_csv')

    ## 4
    my_print.printY('\nTest 4: read_csv()')
    path = 'mylib_test/csvtest.csv'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, logsel)
    string = my_csv_treatment1.read_csv()
    loggerm.info('read_csv' + ' ' + string)

    ## 5
    my_print.printY('\nTest 5: read_csv()')
    string = my_csv_treatment.read_csv()
    loggerm.info('read_csv' + ' ' + string)

    ## 6
    my_print.printY('\nTest 6: check_csv()')
    key = 'ファイル名'
    value = 'sample0.png'
    flg = my_csv_treatment1.check_csv(key, value)
    loggerm.info('check_csv' + ' ' + str(flg))

    ## 7
    my_print.printY('\nTest 7: read_list2_csv()')
    path = 'mylib_test/dic_key.txt'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, logsel)
    befor, after = my_csv_treatment1.read_list2_csv()
    loggerm.info('read_list2_csv' + '\n' + '(befor)\n' + str(befor) + '\n(after)\n' + str(after))

    ## 8
    my_print.printY('\nTest 8: read_list2_csv()')
    path = 'mylib_test/dic_title.txt'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, logsel)
    befor, after = my_csv_treatment1.read_list2_csv()
    loggerm.info('read_list2_csv' + '\n' + '(befor)\n' + str(befor) + '\n(after)\n' + str(after))

    ## 9
    my_print.printY('\nTest 9: read_list2_csv()')
    path = 'mylib_test/dic_company.txt'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, logsel)
    befor, after = my_csv_treatment1.read_list2_csv()
    loggerm.info('read_list2_csv' + '\n' + '(befor)\n' + str(befor) + '\n(after)\n' + str(after))

    ## 10
    my_print.printY('\nTest 10: read_list2_csv()')
    path = 'mylib_test/dic_date.txt'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, logsel)
    befor, after = my_csv_treatment1.read_list2_csv()
    loggerm.info('read_list2_csv' + '\n' + '(befor)\n' + str(befor) + '\n(after)\n' + str(after))

    ## 11
    my_print.printY('\nTest 11: read_list2_csv()')
    path = 'mylib_test/facelist.txt'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, logsel)
    fname, name = my_csv_treatment1.read_list2_csv()
    loggerm.info('read_list2_csv' + '\n' + '(fname)\n' + str(fname) + '\n(name)\n' + str(name))

    ## 12
    my_print.printY('\nTest 12: read_csv_row_value()')
    path = 'mylib_test/attend.csv'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, logsel)
    key = 'Name'
    column = 'Time'
    value = '桶川秀次'
    s = my_csv_treatment1.read_csv_row_value(key, value, column)
    if s is not None:
        loggerm.info(f"read_csv_row_value: value='{value}' 'Time'= {s}")
    else:
        loggerm.info(f"read_csv_row_value: unknown value='{value}'")
    value = '井筒政弘'
    s = my_csv_treatment1.read_csv_row_value(key, value, column)
    if s is not None:
        loggerm.info(f"read_csv_row_value: value='{value}' 'Time'= {s}")
    else:
        loggerm.info(f"read_csv_row_value: unknown value='{s}'")

    ## 13
    my_print.printY('\nTest 13: read_list3_csv()')
    path = 'mylib_test/sample_csv_utf8sig.csv'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, logsel)
    list1, list2, list3 = my_csv_treatment1.read_list3_csv()
    loggerm.info('read_list3_csv' + '\n' + '(list1)\n' + str(list1) + '\n(list2)\n' + str(list2) + '\n(list3)\n' + str(list3))

    ## 14
    my_print.printY('\nTest 14: write_list3_csv()')
    path = 'mylib_test/sample_csv_utf8sig.csv'
    encoding = 'utf_8_sig'
    list2[1] ='いづつ'
    my_csv_treatment1.write_list3_csv(list1, list2, list3)
    loggerm.info('writw_list3_csv' + '\n' + '(list1)\n' + str(list1) + '\n(list2)\n' + str(list2) + '\n(list3)\n' + str(list3))

