# -*- coding: utf-8 -*-
##------------------------------------------
##   My Library File treatment  Ver 0.05
##
##               2022.06.29 Masahiro Izutsu
##------------------------------------------
## my_file.py        (mylib_file.py  Ver 0.04)
##      Ver 0.05    ログ出力変更  2022.06.29

import os, sys
import fitz
import glob
import cv2
from pathlib import Path
import shutil
import imghdr
import my_logging
import my_dialog

class FileTreatment:
    # 初期化
    #   logsel: ログ出力選択
    def __init__(self, logsel = False):
        self.logger = my_logging.get_module_logger_sel(__name__, logsel)

    # PDFファイルから画像ファイルを抽出する
    #   XXXXXXXX.pdf → XXXXXXXX/XXXXXXXX_ページ数_番号.イメージ拡張子
    def pdf2img(self, fname):
        dstdir = os.path.splitext(fname)[0]
        dstn = os.path.basename(dstdir)
        topn = Path(dstdir).stem
        os.makedirs(dstdir, exist_ok=True)
        with fitz.open(fname) as doc:
            for i, page in enumerate(doc):
                for j, img in enumerate(page.get_images()):
                    x = doc.extract_image(img[0])
                    name = os.path.join(dstdir, f"{topn:s}_{i:04}_{j:02}.{x['ext']}")
                    with open(name, "wb") as ofh:
                        ofh.write(x['image'])
                        fsname = os.path.basename(name)
                        self.logger.debug(f'Image file: {dstn}/{fsname}')
        return dstdir

    # ファイルパスからファイル一覧を得る
    def get_file_list(self, dirpass):
        result = sorted(glob.glob(dirpass))
        self.logger.debug(f'File list: {result}')
        return result

    # ファイルパスから拡張子で選択したファイル一覧を得る
    def get_file_list_sel(self, dirpass, sel_ext):
        flist = sorted(glob.glob(dirpass))
        result = []
        for name in flist:
            for ext in sel_ext:
                if ext == os.path.splitext(name)[1]:
                    result.append(name)

        self.logger.debug(f'Select list: {result}')
        return result

    # ファイルを移動する XXXX/XXXXX.xxx ←→  XXXX-err/XXXXX.xxx
    def move_file(self, filename):
        fname = os.path.basename(filename)
        fpath = os.path.dirname(filename)
        if fpath[-4:] == '-err':
            tgtpath = fpath[0:-4]
            if not os.path.exists(tgtpath):
                os.mkdir(tgtpath)
            tgt = tgtpath + '/' + fname
        else:
            tgtpath = fpath + '-err'
            if not os.path.exists(tgtpath):
                os.mkdir(tgtpath)
            tgt = tgtpath + '/' + fname
        shutil.move(filename,tgt)
        self.logger.debug(f'Move file {filename} → {tgt}')

    # -err フォルダのチェック
    def ckeck_erfolder(self, filename):
        fname = os.path.basename(filename)
        fpath = os.path.dirname(filename)
        errf = True if fpath[-4:] == '-err' else False
        return errf

    # ファイル末尾のチェック
    def ckeck_filetail(self, filename, ftail):
        fname = os.path.basename(filename)
        f_base = os.path.splitext(fname)[0]
        n = len(ftail)
        f = True if f_base[-n:] == ftail else False
        return f

    # ファイルを保存する XXXXX.xxx ←→  'dname'/XXXXX.xxx
    def save_file(self, filename, dname = 'org'):
        fname = os.path.basename(filename)
        fpath = os.path.dirname(filename)
        tgtpath = fpath + '/' + dname
        if not os.path.exists(tgtpath):
            os.mkdir(tgtpath)
        tgt = tgtpath + '/' + fname
        shutil.move(filename,tgt)
        self.logger.debug(f'Save file {filename} → {tgt}')

    # イメージファイルを時計回りに90°回転
    def img_rotate90(self, filelist):
        for fname in filelist:
            self.img_rotate90_file(fname)

    def img_rotate90_file(self, filename):
        img = cv2.imread(filename)
        img_rotate_90_clockwise = cv2.rotate(img, cv2.ROTATE_90_CLOCKWISE)
        cv2.imwrite(filename, img_rotate_90_clockwise)
        fsname = os.path.basename(filename)
        self.logger.debug(f'Image rotate_90: {fsname}')

    # イメージファイルを180°回転
    def img_rotate180(self, filelist):
        for fname in filelist:
            self.img_rotate180_file(fname)

    def img_rotate180_file(self, filename):
        img = cv2.imread(filename)
        img_rotate_180 = cv2.rotate(img, cv2.ROTATE_180)
        cv2.imwrite(filename, img_rotate_180)
        fsname = os.path.basename(filename)
        self.logger.debug(f'Image rotate_180: {fsname}')

    # イメージファイルを時計回りに270°回転
    def img_rotate270(self, filelist):
        for fname in filelist:
            self.img_rotate270_file(fname)

    def img_rotate270_file(self, filename):
        img = cv2.imread(filename)
        img_rotate_90_counterclockwise = cv2.rotate(img, cv2.ROTATE_90_COUNTERCLOCKWISE)
        cv2.imwrite(filename, img_rotate_90_counterclockwise)
        fsname = os.path.basename(filename)
        self.logger.debug(f'Image rotate_270: {fsname}')

    # 画像の種類を判別する
    #   戻り値: 'jeg''png'... 画像ファイル
    #           'None'        画像ファイル以外 (動画ファイル)
    #           'NotFound'    ファイルが存在しない
    def is_pict(self, filename):
        try:
            imgtype = imghdr.what(filename)
        except FileNotFoundError as e:
            imgtype = 'NotFound'
        return str(imgtype)


#-----Test routine-----
# $ python3 my_file.py {option:0/1/2/3/4/5/6}
#
if __name__ == "__main__":
    logsel = 0
    args = sys.argv
    sel = '0' if len(args) <= 1 else args[1]
    my_file_Treatment = FileTreatment(logsel)

    if sel == '0':
        # PDFファイルからイメージファイルを抽出
        dirname = my_dialog.select_dir('「pdf2img」')
        if len(dirname) > 0:
            pdflist = my_file_Treatment.get_file_list(dirname + '/*.pdf')
            for name in pdflist:
                dist = my_file_Treatment.pdf2img(name)

    elif sel == '1':
        # イメージファイルのリスト
        dirname = my_dialog.select_dir('「get_file_list_sel」')
        if len(dirname) > 0:
            imglist = my_file_Treatment.get_file_list(dirname + '/*')
            sel_ext = {'.bmp', '.png', '.jpeg', '.jpg', '.tif'}
            sellist = my_file_Treatment.get_file_list_sel(dirname + '/*', sel_ext)

    elif sel == '2':
        # イメージファイルを回転
        dirname = my_dialog.select_dir('「img_rotate270」')
        if len(dirname) > 0:
            imglist = my_file_Treatment.get_file_list(dirname + '/*')
            my_file_Treatment.img_rotate270(imglist)

    elif sel == '3':
        # ファイルを移動する XXXX/XXXXX.xxx ←→  XXXX-err/XXXXX.xxx
        filename = my_dialog.select_image_file("「move_file」")
        if len(filename) > 0:
            ret = my_file_Treatment.move_file(filename)
            print(my_file_Treatment.ckeck_erfolder(filename))

    elif sel == '4':
        # ファイルを保存する XXXXX.xxx ←→  'dname'/XXXXX.xxx
        filename = my_dialog.select_image_file("「save_file」")
        if len(filename) > 0:
            my_file_Treatment.save_file(filename, 'tilt')

    elif sel == '5':
        # ファイルの末尾を調べる
        filename = my_dialog.select_image_file("「ckeck_filetail」")
        if len(filename) > 0:
            print(my_file_Treatment.ckeck_filetail(filename, '_tilt'))

    elif sel == '6':
        # 画像の種類を判別する
        filename = my_dialog.select_image_movie_file("「is_pict」")
        if len(filename) > 0:
            s = my_file_Treatment.is_pict(filename)
            print(f"'{filename}' is '{s}' file")

